#!/usr/bin/env python
# -*- coding: cp1252 -*-

from Bio import SeqIO
import sys,re
import csv
from Bio.Seq import Seq

# Function finding the positions of the potential stop codons
def potentialstop(sequence):
	listpos=[]
	
# list of env codon (codon 1) to have a potential stops on the frame -2. example: codon 1 + tyr => potential stop
	#                               Ala Tyr     Ala Tyr
	#       env                     ATA TAC  => ATT TAC
	#   frame-2                       T AT        A AT
	#                                 Tyr         stop
	listepottyr=["ATA","ACA","ACG","TCA","TCG","CTA","CTG","CCA","CCG","CGA","CGG","GTA","GTG","GCA","GCG","GGA","GGG"]
	listepothisgln=["AAC","ATC","ATA","ACC","ACA","ACG","AGC","TAC","TTC","TCC","TGC","CAC","CTC","CTA","CTG","CCC","CGC","CGA","CGG","GAC","GTC","GTA","GTG","GCC","GCA","GCG","GGC","GGA","GGG"]

	for i in range(2553,3825,3):
		# codon = tyr
		if sequence[i:i+2]=='TA' or sequence[i:i+2]=='TA' :
			
			# no gap between codons
			if not '-' in sequence[i-3:i]:	
				if sequence[i-3:i] in listepottyr:
					listpos.append(i+1)
			# gaps between codons
			else:
				x=i-3
				while sequence[x]=='-':
					x-=3
				if sequence[x:x+3] in listepottyr:
					listpos.append(i+1)
	
						
		else:	
			# codon = his/gln				
			if sequence[i:i+2]=='CA' :
				# no gap between codons
				if not '-' in sequence[i-3:i]:
					if sequence[i-3:i] in listepothisgln:
						listpos.append(i+1)
						
				# gaps between codons
				else:
					x=i-3
					while sequence[x]=='-':
						x-=3
					if sequence[x:x+3] in listepothisgln:
						listpos.append(i+1)

	return listpos


# Function creating the csv file
def writecsv(meta):
	c = csv.writer(open("seq_analyses.csv", "wb"), delimiter='\t')
	c.writerow(["accession number","subtype", "patid","Weigth","Start","Terminal Stop","# Stop", "Stop positions", "early Stop"," alternative Start","length","ASP ORF","# potential stop","potential Stop positions","seq phylo"])

	for acc in meta.keys():
		c.writerow(meta[acc])








# Principal program : generate csv file.
# Available items (for each sequence) are:
# accession number; subtype; PATid; weight; start presence ;  terminal stop presence;  # stop; stop positions; early stop presence; alternative start presence; length; ASP ORF presence; # potentials stops; potential stop position; presence in the phylogeny


# 1) recovery sequences 
try:
	handle = open("./envnucl.fasta","rU")
except IOError: 
	print "The file envnucl.fasta is not present in the directory. "
	sys.exit(1)

seq={}
for record in SeqIO.parse(handle, "fasta") :
	seq[record.id]=str(record.seq)
handle.close()


# 2) Los Alamos metadata:
# accession; subtype; Patid
try:
	reader = csv.reader(open("metadata.csv","rU"), delimiter=';')
except IOError: 
	print "The file metadonnee_base.csv is not present in the directory. "
	sys.exit(1)

meta={}
for row in reader:
	if row[5] in seq.keys():
		# key= accession number;
		# [acc,subtype, patid]
		meta[row[5]]=[row[5],row[4],row[1]]


# 3) weigth
patid={}
for acc in meta.keys():
	if meta[acc][2] in patid.keys():
		patid[meta[acc][2]].append(acc)
	else:
		patid[meta[acc][2]]=[acc]
for listacc in patid.values():
	for acc in listacc:
		meta[acc].append(1./len(listacc))
		

# 4) start presence ;  terminal stop presence;  # stops; stop positions;  early stop presence; alternative start presence; length; ASP ORF presence; # potential stop; potential stop positions;
for acc in meta.keys():
	sequence=seq[acc]

	posstop=[]
	
	
	# ! gaps may be present in stop
	posstop=posstop+[a.start() for a in re.finditer('T-*TA',sequence,re.I)]
	posstop=posstop+[b.start() for b in re.finditer('C-*TA',sequence,re.I)]
	posstop=posstop+[c.start() for c in re.finditer('T-*CA',sequence,re.I)]

	stop=[]
	for pos in posstop:
		if pos%3==2 and pos>=2549 and pos<=3824:
			stop.append(pos)

	# ! gaps may be present in start
	posstart=[d.start() for d in re.finditer('C-*AT',sequence,re.I)]
	start=[]
	for pos in posstart:
		if pos%3==2 and pos>=2549 and pos<=3824:
			start.append(pos)

	# start presence 
	if 3821 in start:
		meta[acc].append(1)
	else:
		meta[acc].append(0)

	# stop terminal presence 
	if 2549 in stop:
		meta[acc].append(1)
		stop.remove(2549)
		
	else:
		meta[acc].append(0)
	
	# nb stop
	meta[acc].append(len(stop))
	# stop positions
	meta[acc].append(stop[:])
	

	
	# early stop presence
	if 3788 in stop:
		meta[acc].append(1)
	else:
		meta[acc].append(0)
		
	# alternative start presence
	if 3734 in start:
		meta[acc].append(1)
	else:
		meta[acc].append(0)
	
	# length
	l=0
	length=[0,0,0]
	if len(start)!=0:
		stop.append(2549)
		
		stop.sort()
		stop.reverse()
		start.sort()
		start.reverse()
		
		for i in range(len(start)):
			x=0
			while -stop[x]<-start[i] and x<len(stop)-1:
				x+=1
			if -stop[x]-(-start[i])>length[2]:
				length[0]=start[i]
				length[1]=stop[x]
				length[2]=-stop[x]-(-start[i])
		
	
		
		l=len(sequence[length[1]:length[0]].replace('-',''))/3

	meta[acc].append(l-1)

	# ASP ORF presence
	if l>150:
		meta[acc].append(1)
	else:
		meta[acc].append(0)
	
	# nb and positions of potential stop codons
	lpotentialstop=potentialstop(sequence)
	
	meta[acc].append(len(lpotentialstop))
	lpotentialstop.sort()
	meta[acc].append(lpotentialstop)



# sequences used in fig 3 phylogeny
try:
	listef=open("acc_phylo.txt","r")
except IOError: 
	print "The file acc_phylo.txt is not present in the directory. "
	sys.exit(1)


liste=listef.readlines()
for i in range(len(liste)):
	accession=liste[i].rstrip()
	liste[i]=accession
listef.close()

for acc in meta.keys():
	if acc in liste:
		meta[acc].append(1)
	else:
		meta[acc].append(0)



writecsv(meta)
		
	


