#!/usr/bin/env python
# -*- coding: cp1252 -*-

from Bio import SeqIO
import sys,re,csv
from Bio.Seq import Seq
import scipy,math
from ete2 import Tree
import os,subprocess
from Bio import AlignIO
from Bio.Alphabet import generic_dna
from Bio.SeqRecord import SeqRecord
from Bio.Align import MultipleSeqAlignment
from Bio import pairwise2


# Retrieving sequence pairs
def readsequences():
	handle = open("listseq.fasta","rU")
	pairseqtot=[]
	for record in SeqIO.parse(handle, "fasta") :
		sequence=str(record.seq).upper()
		pairseqtot.append(sequence)
	handle.close()
	return pairseqtot

# Retrieving position of ASP region and ASP region without RRE
def readcsvpos(posASP,posASPRRE):
	with open('listpos.txt', 'rb') as csvfile:
		spamreader = csv.reader(csvfile, delimiter=' ')
		for row in spamreader:
			posASP.append(int(row[0])+1)
			posASP.append(int(row[2])+1)
			posASPRRE.append(int(row[0])+1)
			posASPRRE.append(int(row[1])+1)
		
# Generation of alignments with only the 3rd bases of synonymous codons
def generatealign(pairseqtot):
	listpairseq3rd=[]
	for x in range(0,len(pairseqtot),2):
		seq1=Seq(pairseqtot[x])
		seq2=Seq(pairseqtot[x+1])
		newseq1m=[]
		newseq2m=[]
		# 3rd base of synonymous sites
		for i in range(0,len(seq1),3):
			s1=str(seq1[i])
			s2=str(seq2[i])
			if not '-' in str(seq1[i:i+3]) and not '-' in str(seq2[i:i+3]):
				if str(seq1[i:i+3].translate()) == str(seq2[i:i+3].translate()):
					newseq1m.append(str(seq1[i+2]))
					newseq2m.append(str(seq2[i+2]))
			
		listpairseq3rd.append(''.join(newseq1m))
		listpairseq3rd.append(''.join(newseq2m))
	return listpairseq3rd

# Parameter estimations 
def parameterestimation(listpairseq3rd):
	# ts: transition; tv: transversion
	ts=0
	tv=0
	nA=0
	nT=0
	nG=0
	nC=0
	lenmean=0
	
	# computation of ts and tv
	for i in range(0,len(listpairseq3rd),2):
		newseq1m=listpairseq3rd[i]
		newseq2m=listpairseq3rd[i+1]

		for i in range(0,len(newseq1m)):
			if newseq1m[i]=='A' or  newseq1m[i]=='G':
				if newseq1m[i] != newseq2m[i]:
					if newseq2m[i]=='A' or  newseq2m[i]=='G':
						ts+=1
					else:
						tv+=1
			else:
				if newseq1m[i] != newseq2m[i]:
					if newseq2m[i]=='T' or  newseq2m[i]=='C':
						ts+=1
					else:
						tv+=1
		# number of each nucleotide
		nA+= float(newseq1m.count('A')+newseq2m.count('A'))
		nT+= float( newseq1m.count('T')+newseq2m.count('T'))
		nG+=  float(newseq1m.count('G')+newseq2m.count('G'))
		nC+=  float(newseq1m.count('C')+newseq2m.count('C'))

	piA=float(nA)/(nA+nT+nG+nC)
	piT=float(nT)/(nA+nT+nG+nC)
	piG=float(nG)/(nA+nT+nG+nC)
	piC=float(nC)/(nA+nT+nG+nC)

	tstv= float(ts)/tv
	alphabeta=tstv*(float((piA*piC+piA*piT+piG*piC+piG*piT))/(piA*piG+piC*piT))
	mu=1/(2*piA*piC+2*piA*piT+2*piG*piC+2*piG*piT+2*piA*piG*alphabeta+2*piC*piT*alphabeta)

	# matrice HKY
	# matrice=[1, mu*piC,mu*alphabeta*piG,mu*piT,mu*piA,1,mu*piG ,mu*alphabeta*piT,mu*piA*alphabeta,mu*piC,1,mu*piT,mu*piA,mu*alphabeta*piC,mu*piG,1]

	# cmd to compute pairwise distances
	modifcom="sed -i '5s/.*/" + str(tstv) + "/' input"
	modif=subprocess.call(modifcom, shell=True)
	modifcom="sed -i '7s/.*/"+str(piA)+" "+str(piC)+" "+str(piG)+" "+str(piT)+"/' input"
	modif=subprocess.call(modifcom, shell=True)
	
	return [piA,piT,piG,piC, mu,alphabeta]
	
	
##################################
###############################
############################
#########################
######################
###################
################
#############
##########
#######
#####
###
#	
	

## Principal program

# read sequences 
pairseqtot=readsequences()

# position of ASP region and ASP region without RRE in pair sequences
posASP=[]
posASPRRE=[]
readcsvpos(posASP, posASPRRE)


# generation of alignments to compute evolutionary parameters
# 3rd base of synonymous site
listpairseq3rd=generatealign(pairseqtot)

# parameters estimation
param=parameterestimation(listpairseq3rd)
piA=param[0]
piT=param[1]
piG=param[2]
piC=param[3]
mu=param[4]
alphabeta=param[5]
cmd = './dnadist < ./input >tmpout'




# list of env codon (codon 1) to have a potential stops on the frame -2. example: codon 1 + tyr => potential stop
	#                               Ala Tyr     Ala Tyr
	#       env                     ATA TAC  => ATT TAC
	#   frame-2                       T AT        A AT
	#                                 Tyr         stop
listepottyr=["ATA","ACA","ACG","TCA","TCG","CTA","CTG","CCA","CCG","CGA","CGG","GTA","GTG","GCA","GCG","GGA","GGG"]
listepothisgln=["AAC","ATC","ATA","ACC","ACA","ACG","AGC","TAC","TTC","TCC","TGC","CAC","CTC","CTA","CTG","CCC","CGC","CGA","CGG","GAC","GTC","GTA","GTG","GCC","GCA","GCG","GGC","GGA","GGG"]


# ASP, ASP without RRE or env-ASP
opt=int(sys.argv[1])

listdist=[]
obs=[]
th=[]
var=[]
pos=[]


# parse sequence pair alignments
for i in range(0,len(listpairseq3rd),2):
	
	
	# dnadist 
	outhandle = open("tmp.phy", "w")
	newseq1=listpairseq3rd[i]
	newseq2=listpairseq3rd[i+1]
	align1 = MultipleSeqAlignment([SeqRecord(Seq(newseq1, generic_dna), id="seq1"), SeqRecord(Seq(newseq2, generic_dna), id="seq2"),])
	AlignIO.write(align1, outhandle, "phylip")
	outhandle.close()
	p = subprocess.call(cmd, shell=True)
	
	# result dnadist
	handle = open("outfile","r")
	dist=handle.read()
	dist=dist.replace("\n","")
	dist=float(dist[-8:])
	handle.close()
	if dist==0:
		dist=1./len(newseq1)

	# computation of probabilities 
	listdist.append(dist)
	pAT=piT*mu*dist
	pAC=piC*mu*dist
	pGT=piT*mu*dist
	pGC=piC*mu*dist
	pCT=piT*mu*dist*alphabeta
	pCA=piA*mu*dist
	pCG=piG*mu*dist
	

	# region to be analyzed
	if opt==1:
		# ASP
		seq1=Seq(pairseqtot[i][posASP[i]:posASP[i+1]])
		seq2=Seq(pairseqtot[i+1][posASP[i]:posASP[i+1]])
		seq1st=pairseqtot[i][posASP[i]:posASP[i+1]]
		seq2st=pairseqtot[i+1][posASP[i]:posASP[i+1]]
		
	if opt==2:	
		# env-ASP
		seq1=Seq(pairseqtot[i][:posASP[i]]+pairseqtot[i][posASP[i+1]:])
		seq2=Seq(pairseqtot[i+1][:posASP[i]]+pairseqtot[i+1][posASP[i+1]:])
		seq1st=pairseqtot[i][:posASP[i]]+pairseqtot[i][posASP[i+1]:]
		seq2st=pairseqtot[i+1][:posASP[i]]+pairseqtot[i+1][posASP[i+1]:]
	
	if opt==3:
		# ASP without RRE
		seq1=Seq(pairseqtot[i][posASPRRE[i]:posASPRRE[i+1]])
		seq2=Seq(pairseqtot[i+1][posASPRRE[i]:posASPRRE[i+1]])
		seq1st=pairseqtot[i][posASPRRE[i]:posASPRRE[i+1]]
		seq2st=pairseqtot[i+1][posASPRRE[i]:posASPRRE[i+1]]
	
	# parse sequence: count observed and expected number of mutation (to obtain a stop codon)
	for i in range(3,len(seq1)-4,3):
		if seq1st[i:i+3]=='TAT' or seq1st[i:i+3]=='TAC' :
			if not '-' in seq1st[i-3:i] and not '-' in seq2[i-3:i] and not '-' in seq2[i:i+3]:
				if str(seq1[i-3:i].translate()) == str(seq2[i-3:i].translate()) and str(seq1[i:i+3].translate()) == str(seq2[i:i+3].translate()):
					if seq1st[i-3:i] in listepottyr:
						pos.append(i)
	
						if seq2st[i-1]=='T' or  seq2st[i-1]=='C':
							obs.append(1)
						else:
							obs.append(0)
						
						if seq1st[i-1]=='A':
							# mutation on env gene to obtain a stop on frame-2 = A->C; A->T
							th.append(pAC+pAT)	
						
						if seq1st[i-1]=='G':
							# mutation on env gene to obtain a stop on frame-2 = G->C; G->T
							th.append(pGT+pGC)
							
							
		if seq1st[i:i+3]=='CAT' or seq1st[i:i+3]=='CAC' or seq1st[i:i+3]=='CAG' or seq1st[i:i+3]=='CAA' :
			if not '-' in seq1st[i-3:i] and not '-' in seq2[i-3:i] and not '-' in seq2[i:i+3] :
				if str(seq1[i-3:i].translate()) == str(seq2[i-3:i].translate()) and str(seq1[i:i+3].translate()) == str(seq2[i:i+3].translate()):
					if seq1st[i-3:i] in listepothisgln:
						pos.append(i)
						
						if seq2st[i-1]=="T":
							obs.append(1)
						else:
							obs.append(0)
						
						if seq1st[i-1]=='A':
							# mutation on env gene to obtain a stop on frame-2 = A->T
							th.append(pAT)				
						if seq1st[i-1]=='G':
							# mutation on env gene to obtain a stop on frame-2 = G->T
							th.append(pGT)	
						if seq1st[i-1]=='C':
							# mutation on env gene to obtain a stop on frame-2 = C->T
							th.append(pCT)
	
		
		
		

# computation of the Z score
sobs=0.0
sth=0.0
n=0.0

for i in range(len(pos)):
	sobs+=obs[i]
	sth+=th[i]
	n+=1



variance=sth*2
e=math.sqrt(variance)
z=(sth-sobs)/e

# computation of the p-value
if z<0:
	pvalue=(1-(scipy.stats.norm.sf(z)))*2
else:
	pvalue=scipy.stats.norm.sf(z)*2

# output
print "# site =",n
print "Z score =",z
print "P-value =", pvalue














