#!/usr/bin/env python
# -*- coding: cp1252 -*-

from Bio import SeqIO
import sys,re,string,csv
from Bio.Seq import Seq
from Bio.Phylo.PAML import codeml
import numpy, math, scipy
from ete2 import Tree
import os,subprocess
from Bio import AlignIO
from Bio.Alphabet import generic_dna
from Bio.SeqRecord import SeqRecord
from Bio.Align import MultipleSeqAlignment
import matplotlib.pyplot as plt
from Bio.SubsMat import MatrixInfo as matlist
from Bio import pairwise2



## generate pairs of aligned sequences


# input 1: env gene nucleotide alignment 
handle = open(sys.argv[1],"rU")
seq={}

for record in SeqIO.parse(handle, "fasta") :
	sequence=str(record.seq).upper()
	seq[record.id]=sequence
handle.close()




# input 2: phylogenetic tree
t = Tree(sys.argv[2])
listleaf=[]
for node in t.iter_descendants("postorder"):
	if node.is_leaf():
		listleaf.append(node.name)
listpair=[]
listpair.append([listleaf[-1],listleaf[0]])
for i in range(0,len(listleaf)-1):
	listpair.append([listleaf[i], listleaf[i+1]])





# generate sequence pair
listpairseq=[]
listpairpos=[]

# outfile 1: fasta file with sequence pairs
outseq= open("listseq.fasta","w")
# outfile 2: txt file with positions of ASP region (with and without RRE) for each sequence pairs 
outpos=open("listpos.txt","w")



for pair in listpair:
	
	seq1tmp=seq[pair[0]]
	seq2tmp=seq[pair[1]]

	seq1=[]
	seq2=[]
	for i in range(0,len(seq1tmp),3):
		if i<len(seq2tmp)-3:
			if not (seq1tmp[i:i+3]=="---" and seq1tmp[i:i+3]==seq2tmp[i:i+3]):
				seq1.append(seq1tmp[i:i+3])
				seq2.append(seq2tmp[i:i+3])
	seq1="".join(seq1)
	seq2="".join(seq2)
	
	# positions of ASP region
	seqASP=seq[pair[0]][2552:3824].replace('-','')
	search="-*".join(seqASP)
	result=re.search(search,seq1)
	
	# positions of ASP region without RRE
	seqRRE=seq[pair[0]][3391:3824].replace('-','')
	search="-*".join(seqRRE)
	resultRRE=re.search(search,seq1)
	
	# write sequences
	outseq.write(">"+str(pair[0])+str(1)+"\n")
	outseq.write(seq1+"\n")
	outseq.write(">"+str(pair[1])+str(2)+"\n")
	outseq.write(seq2+"\n")
	
	# write positions (begin ASP; begin ASP without RRE; end ASP)
	outpos.write(str(result.start())+" "+str(resultRRE.start())+" "+str(result.end())+"\n")
	
	

outseq.close()
outpos.close()







